@@ -181,7 +181,7 @@ totem_display_is_local (void)
 
 	/* Get to the character after the colon */
 	work++;
-	if (work == NULL)
+	if (*work == '\0')
 		return TRUE;
 
 	if (sscanf (work, "%d.%d", &display, &screen) != 2)
@@ -350,25 +350,3 @@ totem_ratio_fits_screen (GdkWindow *video_window, int video_width,
 	return TRUE;
 }
 
-char *
-totem_resolve_relative_link (const char *old_mrl, const char *new_mrl)
-{
-	char *ret, *tmp, *l;
-
-	g_return_val_if_fail (new_mrl != NULL, NULL);
-	g_return_val_if_fail (old_mrl != NULL, NULL);
-
-	/* if new mrl is already absolute, just return it as it is */
-	if (strstr (new_mrl, "://") != NULL)
-		return g_strdup (new_mrl);
-
-	tmp = g_strdup (old_mrl);
-	if ((l = strrchr (tmp, '/')))
-		*l = '\0';
-
-	ret = g_strconcat (tmp, "/", new_mrl, NULL);
-	g_free (tmp);
-
-	return ret;
-}
-
