@@ -275,6 +275,9 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 		/* This is "Title 3", where title is a DVD title
 		 * Note: NOT a DVD chapter */
 		return g_strdup_printf (_("Title %d"), (int) g_strtod (mrl + 6, NULL));
+	} else if (g_str_has_prefix (mrl, "dvb://") != FALSE) {
+		/* This is "BBC ONE(BBC)" for "dvb://BBC ONE(BBC)" */
+		return g_strdup (mrl + 6);
 	}
 
 	file = g_file_new_for_uri (mrl);
@@ -458,19 +461,40 @@ drop_cb (GtkWidget        *widget,
 void
 playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
-	char *subtitle;
+	char *subtitle, *current, *path;
 	GList *l;
+	GFile *file, *dir;
 	TotemPlaylistStatus playing;
 	GtkTreeIter iter;
 
-	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), NULL);
+	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (l);
+
+	/* Look for the directory of the current movie */
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    FILENAME_COL, &current,
+			    -1);
+
+	if (current == NULL)
+		return;
+
+	path = NULL;
+	file = g_file_new_for_commandline_arg (current);
+	dir = g_file_get_parent (file);
+	g_object_unref (file);
+	if (dir != NULL) {
+		path = g_file_get_path (dir);
+		g_object_unref (dir);
+	}
+
+	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
+	g_free (path);
 
 	if (subtitle == NULL)
 		return;
 
-	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
-
 	gtk_tree_model_get (playlist->priv->model, &iter,
 			    PLAYING_COL, &playing,
 			    -1);
@@ -492,16 +516,15 @@ void
 playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
 	GList *l;
-	GtkTreePath *path;
 	GtkClipboard *clip;
 	char *url;
 	GtkTreeIter iter;
 
 	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection,
 			NULL);
-	path = l->data;
-
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (l);
 
 	gtk_tree_model_get (playlist->priv->model,
 			&iter,
@@ -517,8 +540,6 @@ playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playli
 	gtk_clipboard_set_text (clip, url, -1);
 	g_free (url);
 
-	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (l);
 }
 
 static gboolean
@@ -1307,24 +1328,8 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 static void
 init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 {
-	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 
-	/* the model */
-	model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLS,
-				G_TYPE_INT,
-				G_TYPE_STRING,
-				G_TYPE_STRING,
-				G_TYPE_BOOLEAN,
-				G_TYPE_STRING,
-				G_TYPE_OBJECT));
-
-	/* the treeview */
-	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
-	g_object_unref (G_OBJECT (model));
-
 	init_columns (GTK_TREE_VIEW (treeview), playlist);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -1527,43 +1532,6 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
-static gboolean
-totem_playlist_compare_with_monitor (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data)
-{
-	GFileMonitor *monitor = (GFileMonitor *) data;
-	GFileMonitor *_monitor;
-	gboolean retval = FALSE;
-
-	gtk_tree_model_get (playlist->priv->model, iter,
-			    FILE_MONITOR_COL, &_monitor, -1);
-
-	if (_monitor == monitor)
-		retval = TRUE;
-
-	if (_monitor != NULL)
-		g_object_unref (_monitor);
-
-	return retval;
-}
-
-static void
-totem_playlist_file_changed (GFileMonitor *monitor,
-			     GFile *file,
-			     GFile *other_file,
-			     GFileMonitorEvent event_type,
-			     TotemPlaylist *playlist)
-{
-	if (event_type == G_FILE_MONITOR_EVENT_DELETED) {
-		char *uri;
-
-		uri = g_file_get_uri (file);
-		totem_playlist_clear_with_compare (playlist,
-						   (ClearComparisonFunc) totem_playlist_compare_with_monitor,
-						   monitor);
-		g_free (uri);
-	}
-}
-
 static void
 totem_playlist_dispose (GObject *object)
 {
@@ -1672,8 +1640,6 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
 	GtkTreeRowReference *ref;
-	GFileMonitor *monitor;
-	GFile *file;
 	int pos;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
@@ -1701,27 +1667,11 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	store = GTK_LIST_STORE (playlist->priv->model);
 
-	/* Get the file monitor */
-	file = g_file_new_for_uri (uri ? uri : mrl);
-	if (g_file_is_native (file) != FALSE) {
-		monitor = g_file_monitor_file (file,
-					       G_FILE_MONITOR_NONE,
-					       NULL,
-					       NULL);
-		g_signal_connect (G_OBJECT (monitor),
-				  "changed",
-				  G_CALLBACK (totem_playlist_file_changed),
-				  playlist);
-	} else {
-		monitor = NULL;
-	}
-
 	gtk_list_store_insert_with_values (store, &iter, pos,
 			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
-			FILE_MONITOR_COL, monitor,
 			-1);
 
 	g_signal_emit (playlist,
